//
//  LiveApi.swift
//  SodaLive
//
//  Created by klaus on 2023/08/09.
//

import Foundation
import Moya

enum LiveApi {
    case roomList(request: GetRoomListRequest)
    case recentVisitRoomUsers
    case getReservations(isActive: Bool)
    case getReservation(reservationId: Int)
    case cancelReservation(request: CancelLiveReservationRequest)
    case getRoomDetail(roomId: Int)
    case makeReservation(request: MakeLiveReservationRequest)
    case enterRoom(request: EnterOrQuitLiveRoomRequest)
    case getTags
    case getRecentRoomInfo
    case createRoom(parameters: [MultipartFormData])
    case startLive(request: StartLiveRequest)
    case cancelRoom(request: CancelLiveRequest)
    case editLiveRoomInfo(roomId: Int, parameters: [MultipartFormData])
}

extension LiveApi: TargetType {
    var baseURL: URL {
        return URL(string: BASE_URL)!
    }
    
    var path: String {
        switch self {
        case .roomList:
            return "/live/room"
            
        case .recentVisitRoomUsers:
            return "/live/room/recent_visit_room/users"
            
        case .getReservations:
            return "/live/reservation"
            
        case .getReservation(let reservationId):
            return "/live/reservation/\(reservationId)"
            
        case .cancelReservation:
            return "/live/reservation/cancel"
            
        case .getRoomDetail(let roomId):
            return "/live/room/detail/\(roomId)"
            
        case .makeReservation:
            return "/live/reservation"
            
        case .enterRoom:
            return "/live/room/enter"
            
        case .getTags:
            return "/live/tag"
            
        case .getRecentRoomInfo:
            return "/live/room/recent-room-info"
            
        case .createRoom:
            return "/live/room"
            
        case .startLive:
            return "/live/room/start"
            
        case .cancelRoom:
            return "/live/room/cancel"
            
        case .editLiveRoomInfo(let roomId, _):
            return "/live/room/\(roomId)"
        }
    }
    
    var method: Moya.Method {
        switch self {
        case .roomList, .recentVisitRoomUsers, .getReservations, .getReservation, .getRoomDetail, .getTags, .getRecentRoomInfo:
            return .get
            
        case .makeReservation, .enterRoom, .createRoom:
            return .post
            
        case .cancelReservation, .startLive, .cancelRoom, .editLiveRoomInfo:
            return .put
        }
    }
    
    var task: Moya.Task {
        switch self {
        case .roomList(let request):
            var parameters = [
                "timezone": request.timezone,
                "status": request.status.rawValue,
                "page": request.page - 1,
                "size": request.size,
            ] as [String : Any]
            
            if let dateString = request.dateString {
                parameters["dateString"] = dateString
            }
            
            return .requestParameters(
                parameters: parameters,
                encoding: URLEncoding.queryString)
            
        case .recentVisitRoomUsers, .getTags, .getRecentRoomInfo:
            return .requestPlain
            
        case .getReservations(let isActive):
            let parameters = [
                "timezone": TimeZone.current.identifier,
                "isActive": isActive
            ] as [String : Any]
            
            return .requestParameters(
                parameters: parameters,
                encoding: URLEncoding.queryString
            )
            
        case .getReservation:
            let parameters = ["timezone": TimeZone.current.identifier] as [String : Any]
            
            return .requestParameters(
                parameters: parameters,
                encoding: URLEncoding.queryString
            )
            
        case .cancelReservation(let request):
            return .requestJSONEncodable(request)
            
        case .getRoomDetail:
            let parameters = ["timezone": TimeZone.current.identifier] as [String: Any]
            return .requestParameters(parameters: parameters, encoding: URLEncoding.queryString)
            
        case .makeReservation(let request):
            return .requestJSONEncodable(request)
            
        case .enterRoom(let request):
            return .requestJSONEncodable(request)
            
        case .createRoom(let parameters):
            return .uploadMultipart(parameters)
            
        case .startLive(let request):
            return .requestJSONEncodable(request)
            
        case .cancelRoom(let request):
            return .requestJSONEncodable(request)
            
        case .editLiveRoomInfo(_, let parameters):
            return .uploadMultipart(parameters)
        }
    }
    
    var headers: [String : String]? {
        return ["Authorization": "Bearer \(UserDefaults.string(forKey: UserDefaultsKey.token))"]
    }
}
